require 'cgi'
require 'paypal/masspay/error'

module Paypal
  class Masspay::ResponseParser
    SUCCESS_ACK           = 'Success'
    SUCCESS_WITH_WARN_ACK = 'SuccessWithWarning'
    SUCCESSFUL_ACKS       = [ SUCCESS_ACK, SUCCESS_WITH_WARN_ACK ]

    def initialize(response_body)
      @response_body = response_body
      @errors = []
    end

    def successful?
      SUCCESSFUL_ACKS.include?(ack)
    end

    def errors
      @errors.tap do |errors|
        parsed_response.select { |k, v| k.match(/^L_ERRORCODE\d+$/) }.keys.count.times do |i|
          errors << Paypal::Masspay::Error.new(
            :code          => error_code_for(i),
            :short_message => short_message_for(i),
            :long_message  => long_message_for(i),
            :severity      => severity_for(i))
        end
      end
    end

    private

    def parsed_response
      if @parsed_response.nil?
        @parsed_response = CGI::parse(@response_body)
        @parsed_response.each { |key, value| @parsed_response[key] = value.first }
      end

      @parsed_response
    end

    def ack
      parsed_response["ACK"]
    end

    def error_code_for(index)
      parsed_response["L_ERRORCODE#{index}"].to_i
    end

    def short_message_for(index)
      parsed_response["L_SHORTMESSAGE#{index}"]
    end

    def long_message_for(index)
      parsed_response["L_LONGMESSAGE#{index}"]
    end

    def severity_for(index)
      parsed_response["L_SEVERITYCODE#{index}"]
    end
  end
end
